//                                                                  
// Copyright  2008-2009 Lidor Systems.
// All Rights Reserved.
//
// This file is part of the "IntegralUI Lists" class Library. 
//                                                                   
// The contents of this file are subject to the IntegralUI ListBox
// License, and may not be used except in compliance with the License.
// A copy of the License should have been installed in the product's
// root installation directory or it can be found at
// http://www.lidorsystems.com/products/IntegralUI/agreement_listbox.aspx.
//                                                             
// This SOFTWARE is provided "AS IS", WITHOUT WARRANTY OF ANY KIND.
// either express or implied. See the License for the specific language 
// governing rights and limitations under the License.                        
//

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using LidorSystems.IntegralUI.Lists;
using LidorSystems.IntegralUI.Collections;
using LidorSystems.IntegralUI.Serialization;

namespace IULBCatalog
{
    public partial class MainForm : Form
    {
        private String assemblyFullPath = Application.ExecutablePath;

        #region Variables
        private String[][] initData = new String[7][];

        private bool _simpleView = false;
        private CustomSortType _sortType = CustomSortType.byName;
        #endregion

        #region Construction
        public MainForm()
        {
            InitializeComponent();
            InitList();
        }
        #endregion

        #region Methods

        #region Events
        private void lbCatalog_ItemObjectClicked(object sender, LidorSystems.IntegralUI.ObjectClickEventArgs e)
        {
            if (e.Object is LinkLabel)
            {
                LinkLabel lLabel = (LinkLabel)e.Object;
//                System.Diagnostics.Process.Start(lLabel.Links[0].LinkData.ToString());
                MessageBox.Show("Hyperlink for " + lLabel.Links[0].LinkData.ToString() + " is clicked");
            }
        }

        private void OnBuyClick(object sender, EventArgs e)
        {
            if (sender is Button)
            {
                Button btn = (Button)sender;
                if (btn.Tag is ListBoxItem)
                    MessageBox.Show("Buy button for " + (btn.Tag as ListBoxItem).Text + " is clicked");
            }
        }
        #endregion

        #region Initialization

        private void InitalizeData()
        {
            initData[0] = new String[7] { "1", "Digital Camera", "Resources.Camera.jpg", "1,569.20", "1,379.00", "5", "Digital Camera" };
            initData[1] = new String[7] { "2", "Computer Case", "Resources.Case.jpg", "35.70", "29.40", "3", "Computer Case" };
            initData[2] = new String[7] { "6", "Headphones", "Resources.HeadPhones.jpg", "0", "23.90", "4", "Headphones" };
            initData[3] = new String[7] { "5", "CellPhone", "Resources.CellPhone.jpg", "345.20", "279.10", "4", "CellPhone" };
            initData[4] = new String[7] { "4", "Mouse", "Resources.Mouse.jpg", "24.50", "19.40", "2", "Mouse" };
            initData[5] = new String[7] { "1", "Megapixel Digital Camera ", "Resources.MPCamera.jpg", "612.59", "547.99", "4", "Megapixel Digital Camera " };
            initData[6] = new String[7] { "3", "Music Keyboard", "Resources.Keyboard.jpg", "923.40", "789.30", "4", "Music Keyboard" };
        }

        private void InitList()
        {
            this.lbCatalog.SuspendUpdate();
            this.lbCatalog.ScrollMode = ScrollMode.ShowControls;

            this.lbCatalog.Items.Clear();

            InitalizeData();

            ListBoxItem item = null;
            for (int i = 0; i < 7; i++)
            {
                item = new ListBoxItem(initData[i][1]);
                item.Content = CreateItemContent(item, i);
                if (i % 3 == 0)
                    item.StateImageIndex = i % 2;
                item.ToolTip = initData[i][1];

                this.lbCatalog.Items.Add(item);
            }

            this.lbCatalog.ResumeUpdate();
        }

        private String CreateItemContent(ListBoxItem item, int index)
        {
            ProductObject obj = new ProductObject();
            obj.Name = initData[index][1];
            obj.ImageFile = initData[index][2];
            obj.ImageIndex = Convert.ToInt32(initData[index][0]);
            obj.OldPrice = Convert.ToDouble(initData[index][3]);
            obj.CurrentPrice = Convert.ToDouble(initData[index][4]);
            obj.Rating = Convert.ToInt32(initData[index][5]);
            obj.Info = initData[index][6];

            return CreateItemContent(item, obj);
        }

        private void CreateItemControls(ListBoxItem item)
        {
            Button btn = new Button();
            btn.Text = "Buy";
            btn.Size = new Size(50, 22);
            btn.Tag = item;
            btn.Click += new EventHandler(OnBuyClick);
            item.Controls.Add(btn);
        }

        private String CreateItemContent(ListBoxItem item, ProductObject obj)
        {
            CreateItemControls(item);

            item.Tag = obj;
            item.TagString = "Name:" + obj.Name + ";";
            item.TagString += "Image:" + obj.ImageFile + ";";
            item.TagString += "ImageIndex:" + obj.ImageIndex.ToString() + ";";
            item.TagString += "OldPrice:" + obj.OldPrice.ToString() + ";";
            item.TagString += "NewPrice:" + obj.CurrentPrice.ToString() + ";";
            item.TagString += "Rating:" + obj.Rating.ToString() + ";";
            item.TagString += "Info:" + obj.Info;

            String content = String.Empty;
            if (_simpleView)
                content = CreateSimpleContent(obj);
            else
                content = CreateExtendedContent(obj);

            return content;
        }

        private ProductObject ReCreateProductObject(String description)
        {
            ProductObject obj = new ProductObject();

            string[] productAtt = description.Split(new char[] { ';' });
            string[] currentAtt = null;

            for (int i = 0; i < productAtt.Length; i++)
            {
                currentAtt = productAtt[i].Split(new char[] { ':' });
                if (currentAtt.Length == 2)
                {
                    switch (currentAtt[0].ToLower())
                    {
                        case "name":
                            obj.Name = currentAtt[1];
                            break;
                        case "image":
                            obj.ImageFile = currentAtt[1];
                            break;
                        case "imageindex":
                            obj.ImageIndex = Convert.ToInt32(currentAtt[1]);
                            break;
                        case "rating":
                            obj.Rating = Convert.ToInt32(currentAtt[1]);
                            break;
                        case "oldprice":
                            obj.OldPrice = Convert.ToDouble(currentAtt[1]);
                            break;
                        case "newprice":
                            obj.CurrentPrice = Convert.ToDouble(currentAtt[1]);
                            break;
                        case "info":
                            obj.Info = currentAtt[1];
                            break;
                    }
                }
            }

            return obj;
        }

        private void UpdateList()
        {
            String content = String.Empty;

            this.lbCatalog.SuspendUpdate();

            foreach (ListBoxItem item in this.lbCatalog.Items)
            {
                if (item.Tag is ProductObject)
                {
                    if (_simpleView)
                        content = CreateSimpleContent((ProductObject)item.Tag);
                    else
                        content = CreateExtendedContent((ProductObject)item.Tag);

                    item.Content = content;
                }
            }

            this.lbCatalog.ResumeUpdate();
        }

        private String CreateSimpleContent(ProductObject obj)
        {
            String content = String.Empty;
            content = "<div style=\"align:middleleft\"><table width=\"100%\"><tr>";
            content += "<td><img index=\"" + obj.ImageIndex + "\"></img></td>";
            content += "<td width=\"60%\">" + obj.Name + "</td>";
            content += "<td><img assemblypath=\"" + assemblyFullPath + "\" resource=\"" + GetRating(obj.Rating) + "\"></img></td>";
            content += "<td width=\"25%\" style=\"align:middleright;textcolor:Red;hovertextcolor:Red\"><b>$" + obj.CurrentPrice.ToString() + "</b></td>";
            content += "<td width=\"15%\"><a href=\"" + obj.Info + "\" style=\"hovertextcolor:Blue;align:middleright\">More</a></td>";
            content += "</tr></table></div>";

            return content;
        }

        private String GetRating(int rating)
        {
            return "Resources." + rating.ToString() + "star.ico";
        }

        private String CreateExtendedContent(ProductObject obj)
        {
            String content = String.Empty;
            content = "<div style=\"align:middleleft\"><table cellpadding=\"2\" width=\"100%\">";
            content += "<tr><td rowspan=\"5\"><img assemblypath=\"" + assemblyFullPath + "\" resource=\"" + obj.ImageFile + "\" style=\"align:middlecenter\"></img></td><td style=\"font:Arial,12,bi;textcolor:DarkBlue\" width=\"75%\">" + obj.Name + "</td><td width=\"25%\"><a href=\"" + obj.Info + "\" style=\"hovertextcolor:Blue;align:middleright\">More Info</a></td></tr>";
            content += "<tr><td>Regular Price: <s>$" + obj.OldPrice.ToString() + "</s><br></br>Promotional Price: $" + obj.CurrentPrice.ToString() + "</td></tr>";
            content += "<tr><td>User Rating: <img assemblypath=\"" + assemblyFullPath + "\" resource=\"" + GetRating(obj.Rating) + "\"></img></td></tr>";
            content += "<tr><td><b style=\"font:Arial,12,b;textcolor:Red;hovertextcolor:Red\">$" + obj.CurrentPrice.ToString() + "</b></td><td style=\"align:middleright\"><control index=\"0\"></control></td></tr>";
            content += "</table></div>";

            return content;
        }
        #endregion

        #region Menu

        #region File
        private void addToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ProductForm pForm = new ProductForm();
            if (pForm.ShowDialog() == DialogResult.OK)
            {
                ListBoxItem item = new ListBoxItem("Item");
                item.Content = CreateItemContent(item, pForm.Product);
                item.ToolTip = item.Text + "\na little longer text than usual.";
                this.lbCatalog.Items.Add(item);
            }
        }

        private void removeToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            if (this.lbCatalog.SelectedItem != null)
            {
                int selIndex = this.lbCatalog.SelectedItem.Index;
                this.lbCatalog.SelectedItem.Remove();

                if (selIndex > this.lbCatalog.Items.Count - 1)
                    selIndex = this.lbCatalog.Items.Count - 1;

                if (selIndex <= this.lbCatalog.Items.Count - 1)
                    this.lbCatalog.SelectedItem = this.lbCatalog.Items[selIndex];
            }
        }

        private void loadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog openDlg = new OpenFileDialog();
            openDlg.Filter = "XML files (*.xml)|*.xml";
            openDlg.InitialDirectory = Application.StartupPath;
            if (openDlg.ShowDialog() == DialogResult.OK)
            {
                this.lbCatalog.SuspendUpdate();

                ListBoxSerializer serializer = new ListBoxSerializer();
                serializer.LoadFromXml(this.lbCatalog, openDlg.FileName);

                foreach (ListBoxItem item in this.lbCatalog.Items)
                    item.Tag = ReCreateProductObject(item.TagString);

                this.lbCatalog.ResumeUpdate();
            }
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SaveFileDialog saveDlg = new SaveFileDialog();
            saveDlg.Filter = "XML files (*.xml)|*.xml";
            saveDlg.AddExtension = true;
            saveDlg.DefaultExt = ".xml";
            saveDlg.InitialDirectory = Application.StartupPath;

            if (saveDlg.ShowDialog() == DialogResult.OK)
            {
                ListBoxSerializer serializer = new ListBoxSerializer();
                serializer.SaveToXml(this.lbCatalog, saveDlg.FileName);
            }
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }
        #endregion

        #region View
        private void simpleToolStripMenuItem_Click(object sender, EventArgs e)
        {
            simpleToolStripMenuItem.Checked = true;
            extendedToolStripMenuItem.Checked = false;
            _simpleView = true;

            UpdateList();
        }

        private void extendedToolStripMenuItem_Click(object sender, EventArgs e)
        {
            simpleToolStripMenuItem.Checked = false;
            extendedToolStripMenuItem.Checked = true;
            _simpleView = false;

            UpdateList();
        }

        private void checkBoxesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            checkBoxesToolStripMenuItem.Checked = !checkBoxesToolStripMenuItem.Checked;
            this.lbCatalog.CheckBoxes = checkBoxesToolStripMenuItem.Checked;
        }

        private void stateImagesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            stateImagesToolStripMenuItem.Checked = !stateImagesToolStripMenuItem.Checked;
            this.lbCatalog.ShowStateImages = stateImagesToolStripMenuItem.Checked;
        }

        private void toolTipToolStripMenuItem_Click(object sender, EventArgs e)
        {
            toolTipToolStripMenuItem.Checked = !toolTipToolStripMenuItem.Checked;
            this.lbCatalog.ShowToolTips = toolTipToolStripMenuItem.Checked;
        }

        private void themedToolStripMenuItem_Click(object sender, EventArgs e)
        {
            themedToolStripMenuItem.Checked = !themedToolStripMenuItem.Checked;
            this.lbCatalog.UseTheme = themedToolStripMenuItem.Checked;
        }

        private void classicToolStripMenuItem_Click(object sender, EventArgs e)
        {
            classicToolStripMenuItem.Checked = true;
            xPToolStripMenuItem.Checked = false;
            vistaToolStripMenuItem.Checked = false;

            this.lbCatalog.VisualStyle = LidorSystems.IntegralUI.Style.VisualStyle.Classic;
        }

        private void xPToolStripMenuItem_Click(object sender, EventArgs e)
        {
            classicToolStripMenuItem.Checked = false;
            xPToolStripMenuItem.Checked = true;
            vistaToolStripMenuItem.Checked = false;

            this.lbCatalog.VisualStyle = LidorSystems.IntegralUI.Style.VisualStyle.XP;
        }

        private void vistaToolStripMenuItem_Click(object sender, EventArgs e)
        {
            classicToolStripMenuItem.Checked = false;
            xPToolStripMenuItem.Checked = false;
            vistaToolStripMenuItem.Checked = true;

            this.lbCatalog.VisualStyle = LidorSystems.IntegralUI.Style.VisualStyle.Vista;
        }
        #endregion

        #region Behavior
        private void noneToolStripMenuItem_Click(object sender, EventArgs e)
        {
            noneToolStripMenuItem.Checked = true;
            ascendingToolStripMenuItem.Checked = false;
            descendingToolStripMenuItem.Checked = false;

            this.lbCatalog.Sorting = SortOrder.None;
        }

        private void ascendingToolStripMenuItem_Click(object sender, EventArgs e)
        {
            noneToolStripMenuItem.Checked = false;
            ascendingToolStripMenuItem.Checked = true;
            descendingToolStripMenuItem.Checked = false;

            this.lbCatalog.ListItemSorter = new CustomItemComparer(SortOrder.Ascending, _sortType);
            this.lbCatalog.Sorting = SortOrder.Ascending;
        }

        private void descendingToolStripMenuItem_Click(object sender, EventArgs e)
        {
            noneToolStripMenuItem.Checked = false;
            ascendingToolStripMenuItem.Checked = false;
            descendingToolStripMenuItem.Checked = true;

            this.lbCatalog.ListItemSorter = new CustomItemComparer(SortOrder.Descending, _sortType);
            this.lbCatalog.Sorting = SortOrder.Descending;
        }

        private void sortByNameToolStripMenuItem_Click(object sender, EventArgs e)
        {
            sortByNameToolStripMenuItem.Checked = true;
            sortByPriceToolStripMenuItem.Checked = false;
            sortByRatingToolStripMenuItem.Checked = false;

            _sortType = CustomSortType.byName;
            this.lbCatalog.ListItemSorter = new CustomItemComparer(this.lbCatalog.Sorting, _sortType);
            this.lbCatalog.Sort();
        }

        private void sortByPriceToolStripMenuItem_Click(object sender, EventArgs e)
        {
            sortByNameToolStripMenuItem.Checked = false;
            sortByPriceToolStripMenuItem.Checked = true;
            sortByRatingToolStripMenuItem.Checked = false;

            _sortType = CustomSortType.byPrice;
            this.lbCatalog.ListItemSorter = new CustomItemComparer(this.lbCatalog.Sorting, _sortType);
            this.lbCatalog.Sort();
        }

        private void sortByRatingToolStripMenuItem_Click(object sender, EventArgs e)
        {
            sortByNameToolStripMenuItem.Checked = false;
            sortByPriceToolStripMenuItem.Checked = false;
            sortByRatingToolStripMenuItem.Checked = true;

            _sortType = CustomSortType.byRating;
            this.lbCatalog.ListItemSorter = new CustomItemComparer(this.lbCatalog.Sorting, _sortType);
            this.lbCatalog.Sort();
        }

        private void threeStateCheckBoxToolStripMenuItem_Click(object sender, EventArgs e)
        {
            threeStateCheckBoxToolStripMenuItem.Checked = !threeStateCheckBoxToolStripMenuItem.Checked;
            if (threeStateCheckBoxToolStripMenuItem.Checked)
                this.lbCatalog.CheckMode = LidorSystems.IntegralUI.CheckMode.ThreeState;
            else
                this.lbCatalog.CheckMode = LidorSystems.IntegralUI.CheckMode.TwoState;
        }

        private void hoverSelectionToolStripMenuItem_Click(object sender, EventArgs e)
        {
            hoverSelectionToolStripMenuItem.Checked = !hoverSelectionToolStripMenuItem.Checked;
            this.lbCatalog.HoverSelection = hoverSelectionToolStripMenuItem.Checked;
        }

        private void wordWrapToolStripMenuItem_Click(object sender, EventArgs e)
        {
            wordWrapToolStripMenuItem.Checked = !wordWrapToolStripMenuItem.Checked;
            this.lbCatalog.WordWrap = wordWrapToolStripMenuItem.Checked;
        }
        #endregion

        #region Help
        private void onlineArticlesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start("www.lidorsystems.com/support/articles/");
        }
        #endregion

        #endregion

        #endregion
    }

    #region ProductObject class
    internal class ProductObject : System.Object
    {
        #region Variables
        private String _name = String.Empty;
        private String _imageFile = "Resources.NoPhoto.jpg";
        private int _imageIndex = 0;
        private double _oldPrice = 0;
        private double _currentPrice = 0;
        private int _rating = 0;
        private String _info = String.Empty;
        #endregion

        #region Construction
        internal ProductObject()
        {
        }
        #endregion

        #region Properties
        internal String Name
        {
            get { return _name; }
            set
            {
                if (_name != value)
                    _name = value;
            }
        }

        internal String ImageFile
        {
            get { return _imageFile; }
            set
            {
                if (_imageFile != value)
                    _imageFile = value;
            }
        }

        internal int ImageIndex
        {
            get { return _imageIndex; }
            set
            {
                if (_imageIndex != value)
                    _imageIndex = value;
            }
        }

        internal double OldPrice
        {
            get { return _oldPrice; }
            set
            {
                if (_oldPrice != value)
                    _oldPrice = value;
            }
        }

        internal double CurrentPrice
        {
            get { return _currentPrice; }
            set
            {
                if (_currentPrice != value)
                    _currentPrice = value;
            }
        }

        internal int Rating
        {
            get { return _rating; }
            set
            {
                if (_rating != value)
                    _rating = value;
            }
        }

        internal String Info
        {
            get { return _info; }
            set
            {
                if (_info != value)
                    _info = value;
            }
        }
        #endregion
    }
    #endregion

    #region CustomItemComparer class
    public enum CustomSortType
    {
        byName,
        byPrice,
        byRating
    }

    public class CustomItemComparer : System.Collections.IComparer
    {
        #region Variables
        private System.Windows.Forms.SortOrder sortOrder = System.Windows.Forms.SortOrder.Ascending;
        private CustomSortType sortType = CustomSortType.byName;
        #endregion

        #region Constructors
        public CustomItemComparer()
        {
            Init(System.Windows.Forms.SortOrder.Ascending, CustomSortType.byName);
        }

        public CustomItemComparer(System.Windows.Forms.SortOrder order, CustomSortType type)
        {
            Init(order, type);
        }

        private void Init(System.Windows.Forms.SortOrder order, CustomSortType type)
        {
            sortOrder = order;
            sortType = type;
        }
        #endregion

        #region IComparer Members
        public int Compare(object x, object y)
        {
            if (sortOrder == System.Windows.Forms.SortOrder.Descending)
            {
                object temp = x;
                x = y;
                y = temp;
            }

            if ((x as ListBoxItem).Tag != null && (x as ListBoxItem).Tag.GetType() == typeof(ProductObject) &&
                (y as ListBoxItem).Tag != null && (y as ListBoxItem).Tag.GetType() == typeof(ProductObject))
            {
                ProductObject firstObject = (ProductObject)(x as ListBoxItem).Tag;
                ProductObject secondObject = (ProductObject)(y as ListBoxItem).Tag;

                if (firstObject != null && secondObject != null)
                {
                    switch (sortType)
                    {
                        case CustomSortType.byName:
                            return String.Compare(firstObject.Name, secondObject.Name);

                        case CustomSortType.byPrice:
                            if ((double)firstObject.CurrentPrice < (double)secondObject.CurrentPrice)
                                return -1;
                            else if ((double)firstObject.CurrentPrice > (double)secondObject.CurrentPrice)
                                return 1;
                            break;

                        case CustomSortType.byRating:
                            if ((int)firstObject.Rating < (int)secondObject.Rating)
                                return -1;
                            else if ((int)firstObject.Rating > (int)secondObject.Rating)
                                return 1;
                            break;
                    }
                }
            }

            return 0;
        }
        #endregion
    }
    #endregion
}