'                                                                  
' Copyright  2008-2009 Lidor Systems.
' All Rights Reserved.
'
' This file is part of the "IntegralUI Lists" class Library. 
'                                                                   
' The contents of this file are subject to the IntegralUI ListBox
' License, and may not be used except in compliance with the License.
' A copy of the License should have been installed in the product's
' root installation directory or it can be found at
' http://www.lidorsystems.com/products/IntegralUI/agreement_listbox.aspx.
'                                                             
' This SOFTWARE is provided "AS IS", WITHOUT WARRANTY OF ANY KIND.
' either express or implied. See the License for the specific language 
' governing rights and limitations under the License.                        
'

Imports LidorSystems.IntegralUI.Lists
Imports LidorSystems.IntegralUI.Serialization

Public Class MainForm

#Region "Variables"
    Private initData As String()() = New String(6)() {}

    Private assemblyFullPath As [String] = Application.ExecutablePath
    Private _simpleView As Boolean = False
    Private _sortType As CustomSortType = CustomSortType.byName
#End Region

#Region "Methods"

#Region "Events"
    Private Sub lbCatalog_ItemObjectClicked(ByVal sender As Object, ByVal e As LidorSystems.IntegralUI.ObjectClickEventArgs) Handles lbCatalog.ItemObjectClicked
        If TypeOf e.[Object] Is LinkLabel Then
            Dim lLabel As LinkLabel = DirectCast(e.[Object], LinkLabel)
            ' System.Diagnostics.Process.Start(lLabel.Links[0].LinkData.ToString()); 
            MessageBox.Show("Hyperlink for " + lLabel.Links(0).LinkData.ToString() + " is clicked")
        End If
    End Sub

    Private Sub OnBuyClick(ByVal sender As Object, ByVal e As EventArgs)
        If TypeOf sender Is Button Then
            Dim btn As Button = DirectCast(sender, Button)
            If TypeOf btn.Tag Is ListBoxItem Then
                MessageBox.Show("Buy button for " + TryCast(btn.Tag, ListBoxItem).Text + " is clicked")
            End If
        End If
    End Sub
#End Region

#Region "Initialization"

    Private Sub MainForm_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        InitList()
    End Sub

    Private Sub InitalizeData()
        initData(0) = New String(6) {"1", "Digital Camera", "Resources.Camera.jpg", "1,569.20", "1,379.00", "5", _
        "Digital Camera"}
        initData(1) = New String(6) {"2", "Computer Case", "Resources.Case.jpg", "35.70", "29.40", "3", _
        "Computer Case"}
        initData(2) = New String(6) {"6", "Headphones", "Resources.HeadPhones.jpg", "0", "23.90", "4", _
        "Headphones"}
        initData(3) = New String(6) {"5", "CellPhone", "Resources.CellPhone.jpg", "345.20", "279.10", "4", _
        "CellPhone"}
        initData(4) = New String(6) {"4", "Mouse", "Resources.Mouse.jpg", "24.50", "19.40", "2", _
        "Mouse"}
        initData(5) = New String(6) {"1", "Megapixel Digital Camera ", "Resources.MPCamera.jpg", "612.59", "547.99", "4", _
        "Megapixel Digital Camera "}
        initData(6) = New String(6) {"3", "Music Keyboard", "Resources.Keyboard.jpg", "923.40", "789.30", "4", _
        "Music Keyboard"}
    End Sub

    Private Sub InitList()
        Me.lbCatalog.SuspendUpdate()
        Me.lbCatalog.ScrollMode = ScrollMode.ShowControls

        Me.lbCatalog.Items.Clear()

        InitalizeData()

        Dim item As ListBoxItem = Nothing
        For i As Integer = 0 To 6
            item = New ListBoxItem(initData(i)(1))
            item.Content = CreateItemContent(item, i)
            If i Mod 3 = 0 Then
                item.StateImageIndex = i Mod 2
            End If
            item.ToolTip = initData(i)(1)

            Me.lbCatalog.Items.Add(item)
        Next

        Me.lbCatalog.ResumeUpdate()
    End Sub

    Private Function CreateItemContent(ByVal item As ListBoxItem, ByVal index As Integer) As String
        Dim obj As New ProductObject()
        obj.Name = initData(index)(1)
        obj.ImageFile = initData(index)(2)
        obj.ImageIndex = Convert.ToInt32(initData(index)(0))
        obj.OldPrice = Convert.ToDouble(initData(index)(3))
        obj.CurrentPrice = Convert.ToDouble(initData(index)(4))
        obj.Rating = Convert.ToInt32(initData(index)(5))
        obj.Info = initData(index)(6)

        Return CreateItemContent(item, obj)
    End Function

    Private Sub CreateItemControls(ByVal item As ListBoxItem)
        Dim btn As New Button()
        btn.Text = "Buy"
        btn.Size = New Size(50, 22)
        btn.Tag = item
        AddHandler btn.Click, AddressOf OnBuyClick
        item.Controls.Add(btn)
    End Sub

    Private Function ReCreateProductObject(ByVal description As String) As ProductObject
        Dim obj As New ProductObject()

        Dim productAtt As String() = description.Split(New Char() {";"c})
        Dim currentAtt As String() = Nothing

        For i As Integer = 0 To productAtt.Length - 1
            currentAtt = productAtt(i).Split(New Char() {":"c})
            If currentAtt.Length = 2 Then
                Select Case currentAtt(0).ToLower()
                    Case "name"
                        obj.Name = currentAtt(1)
                        Exit Select
                    Case "image"
                        obj.ImageFile = currentAtt(1)
                        Exit Select
                    Case "imageindex"
                        obj.ImageIndex = Convert.ToInt32(currentAtt(1))
                        Exit Select
                    Case "rating"
                        obj.Rating = Convert.ToInt32(currentAtt(1))
                        Exit Select
                    Case "oldprice"
                        obj.OldPrice = Convert.ToDouble(currentAtt(1))
                        Exit Select
                    Case "newprice"
                        obj.CurrentPrice = Convert.ToDouble(currentAtt(1))
                        Exit Select
                    Case "info"
                        obj.Info = currentAtt(1)
                        Exit Select
                End Select
            End If
        Next

        Return obj
    End Function

    Private Function CreateItemContent(ByVal item As ListBoxItem, ByVal obj As ProductObject) As String
        CreateItemControls(item)

        item.Tag = obj
        item.TagString = "Name:" + obj.Name + ";"
        item.TagString += "Image:" + obj.ImageFile + ";"
        item.TagString += "ImageIndex:" + obj.ImageIndex.ToString() + ";"
        item.TagString += "OldPrice:" + obj.OldPrice.ToString() + ";"
        item.TagString += "NewPrice:" + obj.CurrentPrice.ToString() + ";"
        item.TagString += "Rating:" + obj.Rating.ToString() + ";"
        item.TagString += "Info:" + obj.Info

        Dim content As String = [String].Empty

        If _simpleView Then
            content = CreateSimpleContent(obj)
        Else
            content = CreateExtendedContent(obj)
        End If

        Return content
    End Function

    Private Sub UpdateList()
        Dim content As String = [String].Empty

        Me.lbCatalog.SuspendUpdate()

        For Each item As ListBoxItem In Me.lbCatalog.Items
            If TypeOf item.Tag Is ProductObject Then
                If _simpleView Then
                    content = CreateSimpleContent(DirectCast(item.Tag, ProductObject))
                Else
                    content = CreateExtendedContent(DirectCast(item.Tag, ProductObject))
                End If

                item.Content = content
            End If
        Next

        Me.lbCatalog.ResumeUpdate()
    End Sub

    Private Function CreateSimpleContent(ByVal obj As ProductObject) As String
        Dim content As String = [String].Empty
        content = "<div style=""align:middleleft""><table width=""100%""><tr>"
        content += "<td><img index=""" & obj.ImageIndex & """></img></td>"
        content += "<td width=""60%"">" & obj.Name & "</td>"
        content += "<td><img assemblypath=""" & assemblyFullPath & """ resource=""" & GetRating(obj.Rating) & """></img></td>"
        content += "<td width=""25%"" style=""align:middleright;textcolor:Red;hovertextcolor:Red""><b>$" & obj.CurrentPrice.ToString() & "</b></td>"
        content += "<td width=""15%""><a href=""" & obj.Info & """ style=""hovertextcolor:Blue;align:middleright"">More</a></td>"
        content += "</tr></table></div>"

        Return content
    End Function

    Private Function GetRating(ByVal rating As Integer) As String
        Return "Resources." + rating.ToString() + "star.ico"
    End Function

    Private Function CreateExtendedContent(ByVal obj As ProductObject) As String
        Dim content As String = [String].Empty
        content = "<div style=""align:middleleft""><table cellpadding=""2"" width=""100%"">"
        content += "<tr><td rowspan=""5""><img assemblypath=""" & assemblyFullPath & """ resource=""" & obj.ImageFile & """ style=""align:middlecenter""></img></td><td style=""font:Arial,12,bi;textcolor:DarkBlue"" width=""75%"">" & obj.Name & "</td><td width=""25%""><a href=""" & obj.Info & """ style=""hovertextcolor:Blue;align:middleright"">More Info</a></td></tr>"
        content += "<tr><td>Regular Price: <s>$" & obj.OldPrice.ToString() & "</s><br></br>Promotional Price: $" & obj.CurrentPrice.ToString() & "</td></tr>"
        content += "<tr><td>User Rating: <img assemblypath=""" & assemblyFullPath & """ resource=""" & GetRating(obj.Rating) & """></img></td></tr>"
        content += "<tr><td><b style=""font:Arial,12,b;textcolor:Red;hovertextcolor:Red"">$" & obj.CurrentPrice.ToString() & "</b></td><td style=""align:middleright""><control index=""0""></control></td></tr>"
        content += "</table></div>"

        Return content
    End Function
#End Region

#Region "Menu"

#Region "File"
    Private Sub addToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles addToolStripMenuItem.Click
        Dim pForm As New ProductForm()
        If pForm.ShowDialog() = DialogResult.OK Then
            Dim item As New ListBoxItem("Item")
            item.Content = CreateItemContent(item, pForm.Product)
            item.ToolTip = item.Text + vbLf & "a little longer text than usual."
            Me.lbCatalog.Items.Add(item)
        End If
    End Sub

    Private Sub removeToolStripMenuItem1_Click(ByVal sender As Object, ByVal e As EventArgs) Handles removeToolStripMenuItem1.Click
        If Me.lbCatalog.SelectedItem IsNot Nothing Then
            Dim selIndex As Integer = Me.lbCatalog.SelectedItem.Index
            Me.lbCatalog.SelectedItem.Remove()

            If selIndex > Me.lbCatalog.Items.Count - 1 Then
                selIndex = Me.lbCatalog.Items.Count - 1
            End If

            If selIndex <= Me.lbCatalog.Items.Count - 1 Then
                Me.lbCatalog.SelectedItem = Me.lbCatalog.Items(selIndex)
            End If
        End If
    End Sub

    Private Sub loadToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles loadToolStripMenuItem.Click
        Dim openDlg As New OpenFileDialog()
        openDlg.Filter = "XML files (*.xml)|*.xml"
        openDlg.InitialDirectory = Application.StartupPath
        If openDlg.ShowDialog() = DialogResult.OK Then
            Me.lbCatalog.SuspendUpdate()

            Dim serializer As New ListBoxSerializer()
            serializer.LoadFromXml(Me.lbCatalog, openDlg.FileName)

            For Each item As ListBoxItem In Me.lbCatalog.Items
                item.Tag = ReCreateProductObject(item.TagString)
            Next

            Me.lbCatalog.ResumeUpdate()
        End If
    End Sub

    Private Sub saveToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles saveToolStripMenuItem.Click
        Dim saveDlg As New SaveFileDialog()
        saveDlg.Filter = "XML files (*.xml)|*.xml"
        saveDlg.AddExtension = True
        saveDlg.DefaultExt = ".xml"
        saveDlg.InitialDirectory = Application.StartupPath

        If saveDlg.ShowDialog() = DialogResult.OK Then
            Dim serializer As New ListBoxSerializer()
            serializer.SaveToXml(Me.lbCatalog, saveDlg.FileName)
        End If
    End Sub

    Private Sub exitToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles exitToolStripMenuItem.Click
        Me.Close()
    End Sub
#End Region

#Region "View"
    Private Sub simpleToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles simpleToolStripMenuItem.Click
        simpleToolStripMenuItem.Checked = True
        extendedToolStripMenuItem.Checked = False
        _simpleView = True

        UpdateList()
    End Sub

    Private Sub extendedToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles extendedToolStripMenuItem.Click
        simpleToolStripMenuItem.Checked = False
        extendedToolStripMenuItem.Checked = True
        _simpleView = False

        UpdateList()
    End Sub

    Private Sub checkBoxesToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles checkBoxesToolStripMenuItem.Click
        checkBoxesToolStripMenuItem.Checked = Not checkBoxesToolStripMenuItem.Checked
        Me.lbCatalog.CheckBoxes = checkBoxesToolStripMenuItem.Checked
    End Sub

    Private Sub stateImagesToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles stateImagesToolStripMenuItem.Click
        stateImagesToolStripMenuItem.Checked = Not stateImagesToolStripMenuItem.Checked
        Me.lbCatalog.ShowStateImages = stateImagesToolStripMenuItem.Checked
    End Sub

    Private Sub toolTipToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles toolTipToolStripMenuItem.Click
        toolTipToolStripMenuItem.Checked = Not toolTipToolStripMenuItem.Checked
        Me.lbCatalog.ShowToolTips = toolTipToolStripMenuItem.Checked
    End Sub

    Private Sub themedToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles themedToolStripMenuItem.Click
        themedToolStripMenuItem.Checked = Not themedToolStripMenuItem.Checked
        Me.lbCatalog.UseTheme = themedToolStripMenuItem.Checked
    End Sub

    Private Sub classicToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles classicToolStripMenuItem.Click
        classicToolStripMenuItem.Checked = True
        xPToolStripMenuItem.Checked = False
        vistaToolStripMenuItem.Checked = False

        Me.lbCatalog.VisualStyle = LidorSystems.IntegralUI.Style.VisualStyle.Classic
    End Sub

    Private Sub xPToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles xPToolStripMenuItem.Click
        classicToolStripMenuItem.Checked = False
        xPToolStripMenuItem.Checked = True
        vistaToolStripMenuItem.Checked = False

        Me.lbCatalog.VisualStyle = LidorSystems.IntegralUI.Style.VisualStyle.XP
    End Sub

    Private Sub vistaToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles vistaToolStripMenuItem.Click
        classicToolStripMenuItem.Checked = False
        xPToolStripMenuItem.Checked = False
        vistaToolStripMenuItem.Checked = True

        Me.lbCatalog.VisualStyle = LidorSystems.IntegralUI.Style.VisualStyle.Vista
    End Sub
#End Region

#Region "Behavior"
    Private Sub noneToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles noneToolStripMenuItem.Click
        noneToolStripMenuItem.Checked = True
        ascendingToolStripMenuItem.Checked = False
        descendingToolStripMenuItem.Checked = False

        Me.lbCatalog.Sorting = SortOrder.None
    End Sub

    Private Sub ascendingToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles ascendingToolStripMenuItem.Click
        noneToolStripMenuItem.Checked = False
        ascendingToolStripMenuItem.Checked = True
        descendingToolStripMenuItem.Checked = False

        Me.lbCatalog.ListItemSorter = New CustomItemComparer(SortOrder.Ascending, _sortType)
        Me.lbCatalog.Sorting = SortOrder.Ascending
    End Sub

    Private Sub descendingToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles descendingToolStripMenuItem.Click
        noneToolStripMenuItem.Checked = False
        ascendingToolStripMenuItem.Checked = False
        descendingToolStripMenuItem.Checked = True

        Me.lbCatalog.ListItemSorter = New CustomItemComparer(SortOrder.Descending, _sortType)
        Me.lbCatalog.Sorting = SortOrder.Descending
    End Sub

    Private Sub sortByNameToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles sortByNameToolStripMenuItem.Click
        sortByNameToolStripMenuItem.Checked = True
        sortByPriceToolStripMenuItem.Checked = False
        sortByRatingToolStripMenuItem.Checked = False

        _sortType = CustomSortType.byName
        Me.lbCatalog.ListItemSorter = New CustomItemComparer(Me.lbCatalog.Sorting, _sortType)
        Me.lbCatalog.Sort()
    End Sub

    Private Sub sortByPriceToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles sortByPriceToolStripMenuItem.Click
        sortByNameToolStripMenuItem.Checked = False
        sortByPriceToolStripMenuItem.Checked = True
        sortByRatingToolStripMenuItem.Checked = False

        _sortType = CustomSortType.byPrice
        Me.lbCatalog.ListItemSorter = New CustomItemComparer(Me.lbCatalog.Sorting, _sortType)
        Me.lbCatalog.Sort()
    End Sub

    Private Sub sortByRatingToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles sortByRatingToolStripMenuItem.Click
        sortByNameToolStripMenuItem.Checked = False
        sortByPriceToolStripMenuItem.Checked = False
        sortByRatingToolStripMenuItem.Checked = True

        _sortType = CustomSortType.byRating
        Me.lbCatalog.ListItemSorter = New CustomItemComparer(Me.lbCatalog.Sorting, _sortType)
        Me.lbCatalog.Sort()
    End Sub

    Private Sub threeStateCheckBoxToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles threeStateCheckBoxToolStripMenuItem.Click
        threeStateCheckBoxToolStripMenuItem.Checked = Not threeStateCheckBoxToolStripMenuItem.Checked
        If threeStateCheckBoxToolStripMenuItem.Checked Then
            Me.lbCatalog.CheckMode = LidorSystems.IntegralUI.CheckMode.ThreeState
        Else
            Me.lbCatalog.CheckMode = LidorSystems.IntegralUI.CheckMode.TwoState
        End If
    End Sub

    Private Sub hoverSelectionToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles hoverSelectionToolStripMenuItem.Click
        hoverSelectionToolStripMenuItem.Checked = Not hoverSelectionToolStripMenuItem.Checked
        Me.lbCatalog.HoverSelection = hoverSelectionToolStripMenuItem.Checked
    End Sub

    Private Sub wordWrapToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles wordWrapToolStripMenuItem.Click
        wordWrapToolStripMenuItem.Checked = Not wordWrapToolStripMenuItem.Checked
        Me.lbCatalog.WordWrap = wordWrapToolStripMenuItem.Checked
    End Sub
#End Region

#Region "Help"
    Private Sub onlineArticlesToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles onlineArticlesToolStripMenuItem.Click
        System.Diagnostics.Process.Start("www.lidorsystems.com/support/articles/")
    End Sub
#End Region

#End Region

#End Region

End Class

#Region "ProductObject class"
Friend Class ProductObject
    Inherits Object
#Region "Variables"
    Private _name As String = [String].Empty
    Private _imageFile As String = "Resources.NoPhoto.jpg"
    Private _imageIndex As Integer = 0
    Private _oldPrice As Double = 0
    Private _currentPrice As Double = 0
    Private _rating As Integer = 0
    Private _info As String = [String].Empty
#End Region

#Region "Construction"
    Friend Sub New()
    End Sub
#End Region

#Region "Properties"
    Friend Property Name() As String
        Get
            Return _name
        End Get
        Set(ByVal value As String)
            If _name <> value Then
                _name = value
            End If
        End Set
    End Property

    Friend Property ImageFile() As String
        Get
            Return _imageFile
        End Get
        Set(ByVal value As String)
            If _imageFile <> value Then
                _imageFile = value
            End If
        End Set
    End Property

    Friend Property ImageIndex() As Integer
        Get
            Return _imageIndex
        End Get
        Set(ByVal value As Integer)
            If _imageIndex <> value Then
                _imageIndex = value
            End If
        End Set
    End Property

    Friend Property OldPrice() As Double
        Get
            Return _oldPrice
        End Get
        Set(ByVal value As Double)
            If _oldPrice <> value Then
                _oldPrice = value
            End If
        End Set
    End Property

    Friend Property CurrentPrice() As Double
        Get
            Return _currentPrice
        End Get
        Set(ByVal value As Double)
            If _currentPrice <> value Then
                _currentPrice = value
            End If
        End Set
    End Property

    Friend Property Rating() As Integer
        Get
            Return _rating
        End Get
        Set(ByVal value As Integer)
            If _rating <> value Then
                _rating = value
            End If
        End Set
    End Property

    Friend Property Info() As String
        Get
            Return _info
        End Get
        Set(ByVal value As String)
            If _info <> value Then
                _info = value
            End If
        End Set
    End Property
#End Region
End Class
#End Region

#Region "CustomItemComparer class"
Public Enum CustomSortType
    byName
    byPrice
    byRating
End Enum

Public Class CustomItemComparer
    Implements System.Collections.IComparer
#Region "Variables"
    Private sortOrder As System.Windows.Forms.SortOrder = System.Windows.Forms.SortOrder.Ascending
    Private sortType As CustomSortType = CustomSortType.byName
#End Region

#Region "Constructors"
    Public Sub New()
        Init(System.Windows.Forms.SortOrder.Ascending, CustomSortType.byName)
    End Sub

    Public Sub New(ByVal order As System.Windows.Forms.SortOrder, ByVal type As CustomSortType)
        Init(order, type)
    End Sub

    Private Sub Init(ByVal order As System.Windows.Forms.SortOrder, ByVal type As CustomSortType)
        sortOrder = order
        sortType = type
    End Sub
#End Region

#Region "IComparer Members"
    Public Function Compare(ByVal x As Object, ByVal y As Object) As Integer Implements IComparer.Compare
        If sortOrder = System.Windows.Forms.SortOrder.Descending Then
            Dim temp As Object = x
            x = y
            y = temp
        End If

        If TryCast(x, ListBoxItem).Tag IsNot Nothing AndAlso TryCast(x, ListBoxItem).Tag.[GetType]().Equals(GetType(ProductObject)) AndAlso TryCast(y, ListBoxItem).Tag IsNot Nothing AndAlso TryCast(y, ListBoxItem).Tag.[GetType]().Equals(GetType(ProductObject)) Then
            Dim firstObject As ProductObject = DirectCast(TryCast(x, ListBoxItem).Tag, ProductObject)
            Dim secondObject As ProductObject = DirectCast(TryCast(y, ListBoxItem).Tag, ProductObject)

            If firstObject IsNot Nothing AndAlso secondObject IsNot Nothing Then
                Select Case sortType
                    Case CustomSortType.byName
                        Return [String].Compare(firstObject.Name, secondObject.Name)

                    Case CustomSortType.byPrice
                        If CDbl(firstObject.CurrentPrice) < CDbl(secondObject.CurrentPrice) Then
                            Return -1
                        ElseIf CDbl(firstObject.CurrentPrice) > CDbl(secondObject.CurrentPrice) Then
                            Return 1
                        End If
                        Exit Select

                    Case CustomSortType.byRating
                        If CInt(firstObject.Rating) < CInt(secondObject.Rating) Then
                            Return -1
                        ElseIf CInt(firstObject.Rating) > CInt(secondObject.Rating) Then
                            Return 1
                        End If
                        Exit Select
                End Select
            End If
        End If

        Return 0
    End Function
#End Region

End Class

#End Region